// Login.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "credguic.h"
#include "Login.h"
#include "verifypw.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Default name of the CICS Client ini file
#define INIFILENAME "\\cicscli\\bin\\cicscli.ini"
#define MAX_LINE_LEN 512

/////////////////////////////////////////////////////////////////////////////
// Login dialog


Login::Login(CWnd* pParent /*=NULL*/)
	: CDialog(Login::IDD, pParent)
{
	//{{AFX_DATA_INIT(Login)
	m_Userid = _T("");
	m_Password = _T("");
	m_System = _T("");
	m_clientname = _T("");
	m_errmsg = _T("");
	//}}AFX_DATA_INIT
}


void Login::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(Login)
	DDX_Text(pDX, IDC_USERID, m_Userid);
	DDV_MaxChars(pDX, m_Userid, 8);
	DDX_Text(pDX, IDC_PASSWORD, m_Password);
	DDV_MaxChars(pDX, m_Password, 8);
	DDX_Text(pDX, IDC_SYSTEM, m_System);
	DDV_MaxChars(pDX, m_System, 8);
	DDX_Text(pDX, IDC_CLNTNAME, m_clientname);
	DDV_MaxChars(pDX, m_clientname, 4);
	DDX_Text(pDX, IDC_ERRMSG, m_errmsg);
	DDV_MaxChars(pDX, m_errmsg, 50);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(Login, CDialog)
	//{{AFX_MSG_MAP(Login)
	ON_BN_CLICKED(ID_LOGON, OnLogon)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Login message handlers

void Login::OnLogon() 

{
	// TODO: Add extra validation here

	// Make sure our data variables are updated first
	UpdateData(TRUE);

	// Variable to store a pointer to the main application
	CCredguicApp* myApp;

	// Pointer to CICS Commarea for the CICS VERIFYPW program
	VERIFYCOMM* pComm;

	// CICS return code from Verify Password
	CString cicsRC = _T("");

	// Get a pointer to the main application object
	myApp = (CCredguicApp*) AfxGetApp();

	// Next, run a CICS transaction to verify the userid and password.

	// First, allocate a CclBuf object and get a pointer to the COMMAREA 
	CclBuf buffer(sizeof(VERIFYCOMM), CclBuf::fixed);
	pComm = (VERIFYCOMM*) buffer.dataArea();

	// Insert the Userid and Password into the COMMAREA and set the version to "00"
	memcpy(pComm->VERSION, "00", sizeof(pComm->VERSION));
	CCredguicApp::move_comm(m_Userid, pComm->CICSUSER, sizeof(pComm->CICSUSER), ALPHA_FIELD);
	CCredguicApp::move_comm(m_Password, pComm->CICSPW, sizeof(pComm->CICSPW), ALPHA_FIELD);
	CCredguicApp::move_comm(m_clientname, pComm->CLIENTNAME, sizeof(pComm->CLIENTNAME), ALPHA_FIELD);
	
	// Build a connection and a flow object
	CclConn connection(m_System, m_Userid, m_Password);
	CclFlow sflow(Ccl::sync);

	// Set the cursor to an hourglass
	myApp->BeginWaitCursor();

	// Run the CICS program to verify the userid and password

	try {	// Catch block for CclClass exceptions
		connection.link(sflow, "VERIFYPW", &buffer);

		// Get the CICS return code from Verify Password
		CCredguicApp::comm_move(cicsRC, pComm->RESP, sizeof(pComm->RESP));

		// Get the Client Name
		CCredguicApp::comm_move(m_clientname, pComm->CLIENTNAME, sizeof(pComm->CLIENTNAME));

		// Check if the userid and password were verified
		if (memcmp(cicsRC, "00000", 5) == 0)
		{
			// Save the userid, password and system in the main application object
			//  so that other dialogs can access the information.
			myApp->cics_user        = m_Userid;
			myApp->cics_pw          = m_Password;
			myApp->cics_system      = m_System;
			myApp->cics_client_name = m_clientname;

			// Clear the current CICS error message
			m_errmsg.Empty();

			// Display the results
			UpdateData(FALSE);

			// Run the parent handler, which will exit the dialog
			CDialog::OnOK();
		}
		else
		{
			m_errmsg = "Userid or Password is invalidx";
		}
	}    
	catch ( CclException& exception ) { // Catch CclClass exceptions
		// On some servers, an AEY7 abend code is normally caused by
		// an invalid userid and password - therefore, if we see
		// this case, treat it as an invalid userid or password
		if ((exception.exCode() == Ccl::transaction) && (myApp->pECI->m_Cclabcode == "AEY7"))
		{
			m_errmsg = "Userid or Password is invalid";
		}
		else
		{
			// Capture the Error text into the message area
			m_errmsg = exception.exCodeText();
			// Tell the user about the error
			// Note - for production systems, the next line should probably
			// be commented out or removed
			AfxMessageBox("Error communicating with CICS", MB_OK);
		}
    } // end catch

	// Return to a normal cursor
	myApp->EndWaitCursor();

	// Display the results
	UpdateData(FALSE);
}

BOOL Login::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	// Make sure our data variables are updated first
	UpdateData(TRUE);

	// get the client name from the ini file, if found
	getClientName(m_clientname);

	// Load the CICS System Name Combo box from the CICSCLI.INI file
	int rc = loadSystemNames();

	if (rc == 1)
	{
		// end this dialog immediately
		EndDialog(IDCANCEL);		
	}
	// Display the results
	UpdateData(FALSE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

////////////////////////////////////////////////////////////////
// Routine to load the CICS System Name Combo box with the
// names of all the CICS servers defined in the CICSCLI.INI
// Client initialization file, and select the default system.
////////////////////////////////////////////////////////////////

int Login::loadSystemNames()

{
	int numberSystems;
	int i = 1;
	int rc = 0;
	CString emsg;

	// Variable to store a pointer to the main application
	CCredguicApp* myApp;

	// Get a pointer to the main application object
	myApp = (CCredguicApp*) AfxGetApp();

	// Get a pointer to the Combo Box
	CComboBox *pSystem = (CComboBox *) GetDlgItem(IDC_SYSTEM);

	// Make sure the Combo Box is empty
	pSystem->ResetContent();

	try {
		// Get the number of Systems from the ECI object
		numberSystems = myApp->pECI->serverCount();

		// Get the individual Server Names and load the Combo Box
		while (i <= numberSystems)
		{
			// Load the value into the Combo Box
			pSystem->AddString(myApp->pECI->serverName(i));

			// Load the instance variable to the default value
			if (i == 1)
			{
				m_System = myApp->pECI->serverName(i);
			}

			// Move on to next system
			i++;
		}

		// Select the first value
		pSystem->SetCurSel(0);
    }    // Catch CclClass exceptions
	catch ( CclException& exception ) {
		// Trace point for debugging
		TRACE("Error getting CICS system names - %s\n",exception.exCodeText);

		// Let the user know what happened
		emsg = "Error getting CICS system names - ";
		emsg += exception.exCodeText();
		AfxMessageBox(emsg, MB_OK);

		// The following line should be removed in a production application
		// It is for diagnosis purposes during testing
		AfxMessageBox(exception.diagnose(), MB_OK);

		// set a return code to indicate an error
		rc = 1;
	}

	return rc;
}

////////////////////////////////////////////////////////////////
// Routine to skip blanks in a string
////////////////////////////////////////////////////////////////

char*  skipblanks(char *temptr)

{
	// skip leading blanks
	while (temptr[0] == ' ')
	{
		temptr++;
	}

	return temptr;
}

////////////////////////////////////////////////////////////////
// Routine to try to find a CICS client name by looking in the
// CICS client initialization file (CICSCLI.INI).  The location
// of the CICS client file is defined at the top of this program
// in the INIFILENAME define statement
////////////////////////////////////////////////////////////////

void Login::getClientName(CString& clientName)

{
	FILE *inifile;
	char iniFileLine[MAX_LINE_LEN];
	char clname[5];
	char* temptr;

	// Clear the client name
	clientName.Empty();
	memset(clname, '\0', sizeof(clname));

	if ((inifile = fopen(INIFILENAME, "r")) != NULL)
	{
		while (1)
		{
			// read the next line in the source file 
			if (fgets(iniFileLine, MAX_LINE_LEN, inifile) == NULL)
			{
				 break;
			}

			// Point to the first non-blank character
			temptr = iniFileLine;
			temptr = skipblanks(temptr);

			// Check for comment or blank lines
			if ((strlen(temptr) > 0) && (iniFileLine[0] != ';'))
			{
				// check for the client keyword
				if (memcmp(temptr, "Client ", 7) == 0)
				{
					// skip past the word client
					temptr += 7;
					// skip any extra blanks
					temptr = skipblanks(temptr);

					// check for an equal sign
					if (temptr[0] == '=')
					{
						// skip the equal sign
						temptr++;
						// skip any extra blanks
						temptr = skipblanks(temptr);

						// check for a client name 
						// An asterisk implies the name will be generated
						// by CICS and no name is present in this file.
						if ((strlen(temptr) > 0) && (temptr[0] != '*'))
						{
							// Get up to four characters of the name
							memcpy(clname, temptr, 4);
							// put the client name in the passed variable
							clientName = clname;
						}

						break;
					}
				}

			}
		}

		fclose(inifile);
	}
}

//////////////////////////////////////
// End of File
//////////////////////////////////////
