// brwComm.cpp  :  implementation file
//

// Standard VC++ include for precompiled headers
#include "stdafx.h"
#include "resource.h"
#include "credguic.h"
#include "brwcomm.h"

// Defines for user COMMAREA lengths
#define STARTCOMMSIZE	0
#define BROWSECOMMSIZE  14
#define BRWSPGM			"NEWCBRW "
#define BROWSETRAN		"BRWS"
#define MENUMAPNAME		"MENU"
#define BROWSEMAPNAME	"BROWSE"

//********************************************************************
// Constructor for the brwComm object
//********************************************************************
// Create a separate object to encapsulate the CICS COMMAREA
brwComm::brwComm() : CObject()

{
	// Initialize all the instance variables
	br_account1  = _T("");
	br_name1     = _T("");
	br_amount1   = _T("");
	br_account2  = _T("");
	br_name2     = _T("");
	br_amount2   = _T("");
	br_account3  = _T("");
	br_name3     = _T("");
	br_amount3   = _T("");
	br_account4  = _T("");
	br_name4     = _T("");
	br_amount4   = _T("");
	co_errormsg  = _T("");
	co_errormsg2 = _T("");
	co_errormsg3 = _T("");
}

////////////////////////////////////////////////////////////////
// This routine is a sample of how to dump out the values of an 
// object, so that the afxdump facility can be used elsewhere in
// an application.
////////////////////////////////////////////////////////////////

void brwComm::Dump(CDumpContext& dc) const

{
	// Call the base class function first, to dump the address
	// of the object
	CObject::Dump(dc);

	// Dump the standard header
	CCredguicApp::dumpStandardHeader((pCOMMHEAD) &brws.comm_head, dc);

	// Next, dump out the internal variables
	dc << "br_acct1  = " << br_account1 << "  ";
	dc << "br_name1  = " << br_name1 << "  ";
	dc << "br_amt1   = " << br_amount1 << "\n";
	dc << "br_acct2  = " << br_account2 << "  ";
	dc << "br_name2  = " << br_name2 << "  ";
	dc << "br_amt2   = " << br_amount2 << "\n";
	dc << "br_acct3  = " << br_account3 << "  ";
	dc << "br_name3  = " << br_name3 << "  ";
	dc << "br_amt3   = " << br_amount3 << "\n";
	dc << "br_acct4  = " << br_account4 << "  ";
	dc << "br_name4  = " << br_name4 << "  ";
	dc << "br_amt4   = " << br_amount4 << "\n";
}


int brwComm::executeStartBrowse(CString& acctno)

{
	// Trace point for debugging
	TRACE("Entering StartBrowse\n");

	// Map data length
	int	maplen = 0;

	// Get a pointer to the main application object
	CCredguicApp* myApp = (CCredguicApp*) AfxGetApp();

	// Set the BMS map fields
	maplen = setMenuMapData(acctno);

	// set the fields in the standard header
	myApp->setStandardHeader(&brws.comm_head, STARTCOMMSIZE, maplen, MENUMAPNAME, BROWSETRAN);

	// End of COMMAREA build

	// Execute the CICS Transaction and update the instance variables
	int rc = myApp->executeCICSTran(BRWSPGM, sizeof(BRWSCOMM), &brws);

	// Check if it worked
	if (rc == Ccl::noError)
	{
		// Get the user data from the COMMAREA, setting the
		// instance variables as appropriate
		getDataFromComm();
	}

	return rc;
}

int brwComm::executeBrowseNext(const char brType)

{
	// Trace point for debugging
	TRACE("Entering BrowseNext\n");

	// Assertion to validate brType is either an 'F' or a 'B'
	ASSERT((brType == 'F') || (brType == 'B'));

	// Map data length
	int	maplen = 0;

	// Get a pointer to the main application object
	CCredguicApp* myApp = (CCredguicApp*) AfxGetApp();

	// Set the browse map data
	maplen = setBrowseMapData(brType);

	// set the fields in the standard header
	myApp->setStandardHeader(&brws.comm_head, BROWSECOMMSIZE, maplen, BROWSEMAPNAME, BROWSETRAN);

	// End of COMMAREA build

	// Execute the CICS Transaction and update the instance variables
	int rc = myApp->executeCICSTran(BRWSPGM, sizeof(BRWSCOMM), &brws);

	// Check if it worked
	if (rc == Ccl::noError)
	{
		// Get the user data from the COMMAREA, setting the
		// instance variables as appropriate
		getDataFromComm();
	}

	return rc;
}

int brwComm::setMenuMapData(CString& acctno)

{
	// Trace point for debugging
	TRACE("Entering setMenuMapData\n");

	// Total length of all input data in map
	short maplen = 0;

	// Clear the data area to nulls
	memset(&brws, '\0', sizeof(BRWSCOMM));

	// set the account number from the instance variable
	CCredguicApp::moveBMSfield(acctno,
							   brws.menu.dfhdgai.keyi,
							   &brws.menu.dfhdgai.keyl,
							   &maplen,
							   sizeof(brws.menu.dfhdgai.keyi),
							   NUMERIC_FIELD);

	return maplen;
}

int brwComm::setBrowseMapData(const char brType)

{
	// Trace point for debugging
	TRACE("Entering setBrowseMapData\n");

	// Total length of all input data in map
	// Basically, there is only a single 1 character input field,
	// so we move the data directly into the map area and set the
	// length to 1
	short maplen = 1;

	// Set the direction to forwards or backwards in the BROWSE map
    brws.browse.dfhdgci.diri = brType;
    brws.browse.dfhdgci.dirl = 1;

	return maplen;
}

void brwComm::getDataFromComm()

{
	// Trace point for debugging
	TRACE("Entering browse getDataFromComm\n");

	// Check for a MENU map
	if (memcmp(brws.comm_head.MAPNAME,"MENU    ",
			   sizeof(brws.comm_head.MAPNAME)) == 0)
	{
		CCredguicApp::comm_move(co_errormsg, brws.menu.dfhdgao.msgo,
								sizeof(brws.menu.dfhdgao.msgo));
	}

	// Check for a CICS SEND TEXT
	if (memcmp(brws.comm_head.MAPNAME,"SENDTEXT",
			   sizeof(brws.comm_head.MAPNAME)) == 0)
	{
		CCredguicApp::comm_move(co_errormsg, brws.comm_head.textarea,
								sizeof(brws.comm_head.textarea));
	} // endif

   if (memcmp(brws.comm_head.MAPNAME,"BROWSE  ",
              sizeof(brws.comm_head.MAPNAME)) == 0)
   {
		CCredguicApp::comm_move(br_account1, brws.browse.dfhdgco.number1o,
								sizeof(brws.browse.dfhdgco.number1o));

		CCredguicApp::comm_move(br_name1, brws.browse.dfhdgco.name1o,
								sizeof(brws.browse.dfhdgco.name1o));

		CCredguicApp::comm_move(br_amount1, brws.browse.dfhdgco.amount1o,
								sizeof(brws.browse.dfhdgco.amount1o));

		CCredguicApp::comm_move(br_account2, brws.browse.dfhdgco.number2o,
								sizeof(brws.browse.dfhdgco.number2o));

		CCredguicApp::comm_move(br_name2, brws.browse.dfhdgco.name2o,
								sizeof(brws.browse.dfhdgco.name2o));

		CCredguicApp::comm_move(br_amount2, brws.browse.dfhdgco.amount2o,
								sizeof(brws.browse.dfhdgco.amount2o));

		CCredguicApp::comm_move(br_account3, brws.browse.dfhdgco.number3o,
								sizeof(brws.browse.dfhdgco.number3o));

		CCredguicApp::comm_move(br_name3, brws.browse.dfhdgco.name3o,
								sizeof(brws.browse.dfhdgco.name3o));

		CCredguicApp::comm_move(br_amount3, brws.browse.dfhdgco.amount3o,
								sizeof(brws.browse.dfhdgco.amount3o));

		CCredguicApp::comm_move(br_account4, brws.browse.dfhdgco.number4o,
								sizeof(brws.browse.dfhdgco.number4o));

		CCredguicApp::comm_move(br_name4, brws.browse.dfhdgco.name4o,
								sizeof(brws.browse.dfhdgco.name4o));

		CCredguicApp::comm_move(br_amount4, brws.browse.dfhdgco.amount4o,
								sizeof(brws.browse.dfhdgco.amount4o));

		CCredguicApp::comm_move(co_errormsg, brws.browse.dfhdgco.msg2o,
								sizeof(brws.browse.dfhdgco.msg2o));

		CCredguicApp::comm_move(co_errormsg2, brws.browse.dfhdgco.msg0o,
								sizeof(brws.browse.dfhdgco.msg0o));

		CCredguicApp::comm_move(co_errormsg3, brws.browse.dfhdgco.msg1o,
								sizeof(brws.browse.dfhdgco.msg1o));
   }
}

//***********************************************
//  End of file
//***********************************************