// inqyComm.cpp  :  implementation file
//

// Standard VC++ include for precompiled headers
#include "stdafx.h"
#include "resource.h"
#include "credguic.h"
#include "inqycomm.h"

// Defines for user COMMAREA lengths
#define ADDCOMMSIZE		7
#define INQUIRECOMMSIZE 0
#define UPDATECOMMSIZE	80
#define INQYPGM "NEWCALL "
#define MENUMAPNAME "MENU"
#define DETAILMAPNAME "DETAIL"
#define INQUIRETRAN "UPDT"
#define CHANGETRAN  "UPDT"
#define ADDTRAN     "ADDS"

//********************************************************************
// Constructor for the inqyComm object
//********************************************************************
// Create a separate object to encapsulate the CICS COMMAREA
inqyComm::inqyComm() : CObject()

{
	// Initialize all the instance variables
	co_acctno    = _T("");
	co_name      = _T("");
	co_addr      = _T("");
	co_phone     = _T("");
	co_date      = _T("");
	co_amount    = _T("");
	co_comment   = _T("");
	co_errormsg  = _T("");

	// Initialize the private variables
	old_account = _T("");
}

////////////////////////////////////////////////////////////////
// This routine is a sample of how to dump out the values of an 
// object, so that the afxdump facility can be used elsewhere in
// an application.
////////////////////////////////////////////////////////////////

void inqyComm::Dump(CDumpContext& dc) const

{
	// Call the base class function first, to dump the address
	// of the object
	CObject::Dump(dc);

	// Dump the standard header
	CCredguicApp::dumpStandardHeader((pCOMMHEAD) &inqy.comm_head, dc);

	// Next, dump out the internal variables
	dc << "account  = " << co_acctno << "\n";
	dc << "name     = " << co_name << "\n";
	dc << "address  = " << co_addr << "\n";
	dc << "phone    = " << co_phone << "\n";
	dc << "date     = " << co_date << "\n";
	dc << "amount   = " << co_amount << "\n";
	dc << "comment  = " << co_comment << "\n";
	dc << "old account = " << old_account << "\n";
}


//////////////////////////////////////////////////////////////////////
//  Sets the Commarea for inquires and executes the CICS transaction
//////////////////////////////////////////////////////////////////////
int inqyComm::executeInquiry()

{
	// Trace point for debugging
	TRACE("Entering executeInquiry\n");

	// Map data length
	int	maplen = 0;

	// Get a pointer to the main application object
	CCredguicApp* myApp = (CCredguicApp*) AfxGetApp();

	// Set the BMS map fields
	maplen = setMenuMapData();

	// set the transaction code 
	// note that we will use the update code, so that we get back
	// a user comm area with the current record status. This allows
	// the user to change the data and then press the update button.
	// set the user COMMAREA length to zero
	myApp->setStandardHeader(&inqy.comm_head, INQUIRECOMMSIZE, maplen, MENUMAPNAME, INQUIRETRAN);

	// End of COMMAREA build

	// Execute the CICS Transaction and update the instance variables
	int rc = myApp->executeCICSTran(INQYPGM, sizeof(INQYCOMM), &inqy);

	// Get the data from the CICS COMMAREA and update the instance variables
	getDataFromComm(rc);

	// remember the initial values in case of an update
	// the application requires the original values in the user COMMAREA
	old_account = co_acctno;

	return rc;
}

//////////////////////////////////////////////////////////////////////
// Routine to handle update requests
//////////////////////////////////////////////////////////////////////

int inqyComm::executeChange()

{
	// Trace point for debugging
	TRACE("Entering executeChange\n");

	// Variables for user COMMAREA length and BMS map data length
	int maplen = 0;

	// Get a pointer to the main application object
	CCredguicApp* myApp = (CCredguicApp*) AfxGetApp();

	// Get a pointer to the commarea
	// set the user input data                   
	// the user program expects binary zeros     
	// in any unmodified fields.  Since the      
	// commarea was initialized to binary zeros  
	// no action is necessary on unmodified      
	// fields.  Fields will be compared to their 
	// original values to see if they have been  
	// changed.                                  
	// To update a record requires two user      
	// interactions.  In the first one, the user 
	// enters the account number and selects     
	// update.  The account information is       
	// displayed, and the oldaccount field is    
	// set as a hidden variable in the HTML      
	// page.  The user then changes the data and 
	// selects update again.  This time, the     
	// fields which have been changed are        
	// updated in the file.  The first decision  
	// is to determine whether this is the first 
	// interaction for this account number or    
	// second.                                   

	// Check if we are updating an account that has already been displayed
	// If the accounts match, then proceed with the update, otherwise
	// display the data for the new account number, as an inquiry
	if (co_acctno.Compare(old_account) == 0)
	{
		// second interaction, try to update the  
		// account data.                          
		maplen = setDetailMapData();

     
		// set the user commarea length 
		// for updates, the host program expects 80 bytes 
		myApp->setStandardHeader(&inqy.comm_head, UPDATECOMMSIZE, maplen, DETAILMAPNAME, CHANGETRAN);
	}
	else
	{
		// first interaction, display the account 
		// information for the user to change.    
		// move the account number to the keyi field 
		maplen = setMenuMapData();

		// set the user commarea length to indicate 
		//the first pass.                          
		// Build the standard header part of the COMMAREA
		myApp->setStandardHeader(&inqy.comm_head, INQUIRECOMMSIZE, maplen, MENUMAPNAME, INQUIRETRAN);
	}

	// End of COMMAREA build

	// Execute the CICS Transaction and update the instance variables
	int rc = myApp->executeCICSTran(INQYPGM, sizeof(INQYCOMM), &inqy);

	// Get the data from the CICS COMMAREA and update the instance variables
	getDataFromComm(rc);

	return rc;
}

//////////////////////////////////////////////////////////////////////
// Routine to handle add requests
// Add requests will be done in two passes, as is required by 
// 3270 version of the program.  The first pass will send in the
// add request and account number using a MENU map, and the second
// will send in the rest of the data using a DETAIL map.
// The CICS application will load the user COMMAREA after the first
// request.
//////////////////////////////////////////////////////////////////////

int inqyComm::executeAdd()

{
	// Trace point for debugging
	TRACE("Entering executeAdd\n");

	// BMS data length
	int maplen = 0;

	// Get a pointer to the main application object
	CCredguicApp* myApp = (CCredguicApp*) AfxGetApp();

	// Set the BMS map fields for the first pass
	maplen = setMenuMapData();

	// Set the standard COMMAREA header for the first pass of an add request
	myApp->setStandardHeader(&inqy.comm_head, INQUIRECOMMSIZE, maplen, MENUMAPNAME, ADDTRAN);

	// Execute the CICS Transaction 
	int rc = myApp->executeCICSTran(INQYPGM, sizeof(INQYCOMM), &inqy);

	// Check if the ECI call worked and we received the right data back
	if ((rc == 0) && (memcmp(inqy.comm_head.NXTRNID, ADDTRAN, sizeof(inqy.comm_head.NXTRNID)) == 0))
	{
		// Trace point for debugging
		TRACE("Entering second phase of add\n");

		// Second pass for adding a record
		// Set the input fields in the COMMAREA
		maplen = setDetailMapData();

		// set the transaction code 
		myApp->setStandardHeader(&inqy.comm_head, ADDCOMMSIZE, maplen, DETAILMAPNAME, ADDTRAN);

		// End of COMMAREA build

		// Execute the CICS Transaction and update the instance variables
		int rc = myApp->executeCICSTran(INQYPGM, sizeof(INQYCOMM), &inqy);

		// Get the data from the CICS COMMAREA and update the instance variables
		getDataFromComm(rc);
	}
	else
	{
		// Get the appropriate error message
		if (rc != 0)
		{
			// Get the ECI error message
			co_errormsg = myApp->cics_message;
		}
		else
		{
			// Do not disturb the data if the first pass didn't work
			// check if we got a menu map back
			if (memcmp(inqy.comm_head.MAPNAME,"MENU    ",
					   sizeof(inqy.comm_head.MAPNAME)) == 0)
			{
				CCredguicApp::comm_move(co_errormsg, inqy.menu.dfhdgao.msgo,
										sizeof(inqy.menu.dfhdgao.msgo));
			}
		}
	}

	return rc;
}

int inqyComm::setDetailMapData()

{
	// Trace point for debugging
	TRACE("Entering setDetailMapData\n");

	// Total length of all input data in map
	short maplen = 0;

	// Set the BMS fields
	CCredguicApp::moveBMSfield(co_acctno,
							   inqy.detail.dfhdgbi.numbi,
							   &inqy.detail.dfhdgbi.numbl,
							   &maplen,
							   sizeof(inqy.detail.dfhdgbi.numbi),
							   NUMERIC_FIELD);

	CCredguicApp::moveBMSfield(co_name,
							   inqy.detail.dfhdgbi.namei,
							   &maplen,
							   &inqy.detail.dfhdgbi.namel,
							   sizeof(inqy.detail.dfhdgbi.namei),
							   ALPHA_FIELD);

	CCredguicApp::moveBMSfield(co_addr,
							   inqy.detail.dfhdgbi.addri,
							   &inqy.detail.dfhdgbi.addrl,
							   &maplen,
							   sizeof(inqy.detail.dfhdgbi.addri),
							   ALPHA_FIELD);

	CCredguicApp::moveBMSfield(co_phone,
							   inqy.detail.dfhdgbi.phonei,
							   &inqy.detail.dfhdgbi.phonel,
							   &maplen,
							   sizeof(inqy.detail.dfhdgbi.phonei),
							   ALPHA_FIELD);

	CCredguicApp::moveBMSfield(co_date,
							   inqy.detail.dfhdgbi.datei,
							   &inqy.detail.dfhdgbi.datel,
							   &maplen,
							   sizeof(inqy.detail.dfhdgbi.datei),
							   ALPHA_FIELD);

	CCredguicApp::moveBMSfield(co_amount,
							   inqy.detail.dfhdgbi.amounti,
							   &inqy.detail.dfhdgbi.amountl,
							   &maplen,
							   sizeof(inqy.detail.dfhdgbi.amounti),
							   ALPHA_FIELD);

	CCredguicApp::moveBMSfield(co_comment,
							   inqy.detail.dfhdgbi.commenti,
							   &inqy.detail.dfhdgbi.commentl,
							   &maplen,
							   sizeof(inqy.detail.dfhdgbi.commenti),
							   ALPHA_FIELD);

	return maplen;
}

int inqyComm::setMenuMapData()

{
	// Trace point for debugging
	TRACE("Entering setMenuMapData\n");

	// Total length of all input data in map
	short maplen = 0;

	// Clear the data area to nulls
	memset(&inqy, '\0', sizeof(INQYCOMM));

	// set the account number from the instance variable
	CCredguicApp::moveBMSfield(co_acctno,
							   inqy.menu.dfhdgai.keyi,
							   &inqy.menu.dfhdgai.keyl,
							   &maplen,
							   sizeof(inqy.menu.dfhdgai.keyi),
							   NUMERIC_FIELD);

	return maplen;
}

void inqyComm::getDataFromComm(int RC)

{
	// Trace point for debugging
	TRACE("Entering getDataFromComm\n");

	if (RC == Ccl::noError)
	{
		// Check for a DETAIL map
		if (memcmp(inqy.comm_head.MAPNAME,"DETAIL  ",
			       sizeof(inqy.comm_head.MAPNAME)) == 0)
		{
			// Populate the dialog field variables
			CCredguicApp::comm_move(co_name, inqy.detail.dfhdgbi.namei,
									sizeof(inqy.detail.dfhdgbi.namei));

			CCredguicApp::comm_move(co_addr, inqy.detail.dfhdgbi.addri,
									sizeof(inqy.detail.dfhdgbi.addri));

			CCredguicApp::comm_move(co_phone, inqy.detail.dfhdgbi.phonei,
									sizeof(inqy.detail.dfhdgbi.phonei));

			CCredguicApp::comm_move(co_date, inqy.detail.dfhdgbi.datei,
									sizeof(inqy.detail.dfhdgbi.datei));

			CCredguicApp::comm_move(co_amount, inqy.detail.dfhdgbi.amounti,
									sizeof(inqy.detail.dfhdgbi.amounti));

			CCredguicApp::comm_move(co_comment, inqy.detail.dfhdgbi.commenti,
									sizeof(inqy.detail.dfhdgbi.commenti));

			//setOldValues();


			if ((inqy.detail.dfhdgbo.msg1o[0] != '\0') &&
				(inqy.detail.dfhdgbo.msg1o[0] != ' '))
			{
				CCredguicApp::comm_move(co_errormsg, inqy.detail.dfhdgbo.msg1o,
										sizeof(inqy.detail.dfhdgbo.msg1o));
			}

			if ((inqy.detail.dfhdgbo.msg3o[0] != '\0') &&
				(inqy.detail.dfhdgbo.msg3o[0] != ' '))
			{
				CCredguicApp::comm_move(co_errormsg, inqy.detail.dfhdgbo.msg3o,
										sizeof(inqy.detail.dfhdgbo.msg3o));
			}
		}

		// Check for a MENU map
		if (memcmp(inqy.comm_head.MAPNAME,"MENU    ",
				   sizeof(inqy.comm_head.MAPNAME)) == 0)
		{
			CCredguicApp::comm_move(co_errormsg, inqy.menu.dfhdgao.msgo,
									sizeof(inqy.menu.dfhdgao.msgo));
		}

		// Check for a CICS SEND TEXT
		if (memcmp(inqy.comm_head.MAPNAME,"SENDTEXT",
				   sizeof(inqy.comm_head.MAPNAME)) == 0)
		{
			CCredguicApp::comm_move(co_errormsg, inqy.comm_head.textarea,
									sizeof(inqy.comm_head.textarea));
		} // endif
	}
	else
	{
		// Get a pointer to the main application object
		CCredguicApp* myApp = (CCredguicApp*) AfxGetApp();

		co_errormsg = myApp->cics_message;
	}
}

//***********************************************
//  End of file
//***********************************************